
# q-expansion of newform 3680.2.f.b, downloaded from the LMFDB on 18 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3680
weight = 2
poly_data = [81, 0, 0, 0, 31, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 0, 40, 0, 0, 0, 1, 0], 63], [[12, 0, 0, 0, 1, 0, 0, 0], 7], [[0, 0, -97, 0, 0, 0, -4, 0], 63], [[0, 63, 0, 40, 0, 0, 0, 1], 63], [[0, 171, 0, -97, 0, 9, 0, -4], 189], [[0, -171, 0, -97, 0, -9, 0, -4], 189], [[0, -360, 0, 217, 0, -9, 0, 7], 189]]

hecke_ring_character_values = [[1151, [1, 0, 0, 0, 0, 0, 0, 0]], [1381, [-1, 0, 0, 0, 0, 0, 0, 0]], [737, [1, 0, 0, 0, 0, 0, 0, 0]], [3041, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 1, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0], [-1, 0, -1, 0, 0, 0, 0, 0], [0, -1, 0, -1, 1, 0, -2, 0], [0, 2, 0, -1, -1, 0, -1, 0], [-2, 0, 1, 0, 0, 1, 0, 1], [0, -4, 0, -1, -1, 0, 1, 0], [1, 0, 0, 0, 0, 0, 0, 0], [0, 2, 0, 2, 0, 0, -2, 0], [-1, 0, -3, 0, 0, -2, 0, 1], [0, 4, 0, -2, 0, 0, 2, 0], [2, 0, 1, 0, 0, -2, 0, -2], [0, -2, 0, -2, -2, 0, 0, 0], [-2, 0, -2, 0, 0, -6, 0, 0], [0, -4, 0, 2, -2, 0, -2, 0], [0, 0, 0, 0, -2, 0, 2, 0], [0, 1, 0, 1, -2, 0, 0, 0], [0, 0, 0, 0, -2, 0, 2, 0], [2, 0, -1, 0, 0, -1, 0, -1], [2, 0, 4, 0, 0, 2, 0, 2], [4, 0, 2, 0, 0, -6, 0, 0], [0, -6, 0, 0, -2, 0, 4, 0], [0, 0, 0, 0, 0, -8, 0, -2], [2, 0, 1, 0, 0, 3, 0, 3], [0, -6, 0, 0, 2, 0, 2, 0], [2, 0, 3, 0, 0, 0, 0, 0], [0, 6, 0, 6, 0, 0, 0, 0], [0, 8, 0, -1, -2, 0, 2, 0], [4, 0, -4, 0, 0, -2, 0, 4], [-6, 0, -4, 0, 0, 4, 0, 4], [0, 8, 0, 2, 4, 0, -2, 0], [3, 0, -1, 0, 0, -4, 0, -1], [0, -10, 0, -4, 4, 0, -8, 0], [0, 0, 0, 3, -2, 0, -6, 0], [2, 0, 5, 0, 0, 3, 0, 3], [0, -6, 0, 0, -4, 0, 4, 0], [0, -2, 0, -5, 2, 0, -2, 0], [-6, 0, 4, 0, 0, -2, 0, -2], [0, 3, 0, 3, -3, 0, -6, 0], [0, 6, 0, -6, 0, 0, 0, 0], [0, 4, 0, -5, 0, 0, 0, 0], [4, 0, -2, 0, 0, -6, 0, 0], [4, 0, 0, 0, 0, -6, 0, 0], [0, -5, 0, -5, -3, 0, 10, 0], [8, 0, 0, 0, 0, 2, 0, -4], [0, -4, 0, -4, 6, 0, 0, 0], [-6, 0, -6, 0, 0, 6, 0, 0], [0, 8, 0, -4, -4, 0, 2, 0], [0, 2, 0, 8, 4, 0, 0, 0], [-2, 0, 4, 0, 0, -2, 0, 4], [-2, 0, -4, 0, 0, -4, 0, -4], [-6, 0, 4, 0, 0, -2, 0, 4], [0, -4, 0, -1, -1, 0, -7, 0], [6, 0, 2, 0, 0, -2, 0, 4], [-6, 0, 5, 0, 0, -8, 0, 4], [0, -12, 0, -6, 2, 0, -2, 0], [5, 0, -1, 0, 0, 6, 0, -3], [0, 8, 0, -4, 0, 0, -6, 0], [-2, 0, -2, 0, 0, 10, 0, 4], [0, 12, 0, 0, 0, 0, -6, 0], [0, -16, 0, 2, 0, 0, -8, 0], [0, -11, 0, 1, -2, 0, 12, 0], [18, 0, 0, 0, 0, -4, 0, -4], [-9, 0, -1, 0, 0, -4, 0, 5], [0, -6, 0, 3, 7, 0, 7, 0], [0, -2, 0, 4, -4, 0, 6, 0], [3, 0, 3, 0, 0, -12, 0, -3], [0, 9, 0, -3, 2, 0, -4, 0], [0, 2, 0, 8, -2, 0, 4, 0], [0, 0, 6, 0, 0, 10, 0, -2], [-16, 0, -4, 0, 0, 4, 0, 4], [8, 0, -8, 0, 0, 0, 0, 0], [0, -2, 0, -2, 8, 0, 0, 0], [0, 3, 0, -9, -5, 0, 2, 0], [-20, 0, -8, 0, 0, -4, 0, -4], [0, -17, 0, -5, 4, 0, -8, 0], [0, 25, 0, 1, -1, 0, 2, 0], [-18, 0, -6, 0, 0, 2, 0, 2], [7, 0, -7, 0, 0, -4, 0, -4], [0, 10, 0, 4, -4, 0, -4, 0], [0, 0, 0, 9, -2, 0, 2, 0], [12, 0, 2, 0, 0, 4, 0, -2], [9, 0, 1, 0, 0, 8, 0, 5], [-17, 0, -3, 0, 0, 2, 0, -1], [0, -14, 0, 1, 8, 0, -4, 0], [3, 0, -3, 0, 0, 12, 0, 0], [4, 0, 4, 0, 0, 6, 0, 12], [0, 14, 0, 2, -6, 0, 8, 0], [16, 0, -4, 0, 0, -6, 0, 6], [0, 10, 0, 4, 0, 0, 10, 0], [-12, 0, 2, 0, 0, -10, 0, 2], [-20, 0, 0, 0, 0, 10, 0, 4], [0, 12, 0, -6, 0, 0, -4, 0], [0, 26, 0, 8, 4, 0, -10, 0], [21, 0, 1, 0, 0, 8, 0, 2], [0, 2, 0, -4, -4, 0, -10, 0], [-6, 0, 2, 0, 0, -4, 0, -10], [0, -6, 0, -6, 4, 0, -10, 0], [0, 16, 0, 4, -8, 0, 4, 0], [0, -2, 0, -5, 0, 0, 12, 0], [0, 0, 0, 0, 4, 0, 14, 0], [0, 18, 0, 6, -2, 0, -10, 0], [0, 0, -6, 0, 0, 2, 0, 8], [0, -1, 0, -1, -15, 0, 6, 0], [2, 0, -4, 0, 0, 12, 0, -6], [0, -3, 0, -3, -10, 0, 4, 0], [-12, 0, -8, 0, 0, 8, 0, 2], [14, 0, 17, 0, 0, 1, 0, 1], [5, 0, 7, 0, 0, 16, 0, -2], [8, 0, -6, 0, 0, 6, 0, 0], [0, 10, 0, 10, -8, 0, 10, 0], [-6, 0, -11, 0, 0, 11, 0, -1], [0, -8, 0, 1, 7, 0, -3, 0], [-8, 0, 2, 0, 0, 0, 0, 0], [6, 0, 0, 0, 0, -4, 0, 2], [0, 8, 0, 8, -12, 0, 12, 0], [-24, 0, -12, 0, 0, -12, 0, -6], [0, 18, 0, -3, -1, 0, 7, 0], [0, -18, 0, 0, -4, 0, 8, 0], [0, -3, 0, -3, 0, 0, -24, 0], [-4, 0, -8, 0, 0, 6, 0, -6], [0, 2, 0, 8, -4, 0, 14, 0], [0, -6, 0, -9, -4, 0, 0, 0], [0, -26, 0, 4, 0, 0, -8, 0], [0, -12, 0, -21, 0, 0, -4, 0], [0, 7, 0, 7, -4, 0, -8, 0], [5, 0, -1, 0, 0, 10, 0, 7], [-6, 0, -11, 0, 0, 6, 0, -6], [0, 4, 0, -2, -6, 0, -6, 0], [0, 2, 0, 8, -6, 0, -8, 0], [7, 0, -5, 0, 0, 0, 0, 0], [-8, 0, -12, 0, 0, 18, 0, 0], [0, 10, 0, 4, 10, 0, -4, 0], [-3, 0, 3, 0, 0, 0, 0, -6], [2, 0, -6, 0, 0, 10, 0, 10], [0, 6, 0, 6, 6, 0, -2, 0], [0, 0, 0, -6, 2, 0, 4, 0], [0, 8, 0, 2, -8, 0, 4, 0], [-26, 0, -5, 0, 0, -6, 0, -6], [0, -24, 0, -6, -6, 0, 12, 0], [0, 6, 0, 18, 4, 0, -4, 0], [12, 0, 4, 0, 0, -12, 0, -12], [0, -12, 0, -6, 6, 0, -6, 0], [0, 14, 0, 2, 2, 0, -8, 0], [20, 0, -4, 0, 0, -18, 0, 0], [0, -10, 0, -7, -5, 0, 3, 0], [-20, 0, -8, 0, 0, -6, 0, 0], [0, 8, 0, 2, 8, 0, -20, 0], [-16, 0, 4, 0, 0, 10, 0, -2], [0, -1, 0, -13, -9, 0, 6, 0], [2, 0, -2, 0, 0, -8, 0, -2], [0, 2, 0, 5, 10, 0, -18, 0], [-4, 0, -6, 0, 0, -4, 0, 2], [0, -16, 0, -10, 0, 0, 10, 0], [10, 0, 12, 0, 0, -8, 0, -2], [-6, 0, -10, 0, 0, -6, 0, 6], [6, 0, -4, 0, 0, -32, 0, 4], [-7, 0, -15, 0, 0, 8, 0, -1], [0, 15, 0, 3, 0, 0, 8, 0], [0, -18, 0, -3, 2, 0, -10, 0], [-6, 0, -9, 0, 0, -15, 0, -3], [20, 0, 2, 0, 0, 18, 0, 0], [0, -5, 0, -5, 7, 0, 6, 0], [31, 0, 3, 0, 0, 16, 0, 7], [-34, 0, 1, 0, 0, 10, 0, -2], [-14, 0, -7, 0, 0, -5, 0, 7], [0, -12, 0, 0, 8, 0, -14, 0], [8, 0, 0, 0, 0, 16, 0, -2], [0, -9, 0, -9, 9, 0, -2, 0], [0, 26, 0, -4, 8, 0, 8, 0], [0, -8, 0, -8, 6, 0, -18, 0], [-6, 0, 0, 0, 0, -4, 0, -4], [18, 0, -1, 0, 0, 9, 0, -3], [18, 0, -7, 0, 0, 9, 0, -3], [0, 0, -2, 0, 0, 10, 0, 10], [0, -4, 0, -16, 4, 0, -4, 0], [0, -6, 0, 12, 0, 0, -12, 0], [30, 0, 13, 0, 0, 22, 0, 10], [0, -8, 0, 7, 4, 0, -12, 0], [-2, 0, 12, 0, 0, -20, 0, 4], [0, 25, 0, 13, -3, 0, 2, 0], [0, 6, 0, -9, -3, 0, 9, 0], [-20, 0, 10, 0, 0, -16, 0, 8], [-6, 0, -3, 0, 0, -24, 0, -12], [0, -12, 0, 3, -2, 0, -2, 0], [0, -10, 0, -7, -1, 0, 3, 0], [0, -2, 0, 10, -8, 0, -18, 0], [2, 0, -5, 0, 0, 16, 0, 4], [-42, 0, -5, 0, 0, 1, 0, 1], [-24, 0, -8, 0, 0, 12, 0, 0], [0, 16, 0, 4, 4, 0, -12, 0], [0, -14, 0, -8, 12, 0, -4, 0], [0, 2, 0, 2, 14, 0, -10, 0], [0, -24, 0, -12, -6, 0, 14, 0], [-25, 0, 3, 0, 0, -8, 0, -5], [-10, 0, -16, 0, 0, 6, 0, 6], [0, 20, 0, 2, 2, 0, 10, 0], [4, 0, -20, 0, 0, 2, 0, 8], [6, 0, -4, 0, 0, -8, 0, -8], [0, 33, 0, 9, 2, 0, -16, 0], [27, 0, 13, 0, 0, 10, 0, 1], [0, -11, 0, 13, 1, 0, -10, 0], [10, 0, -2, 0, 0, 24, 0, 6], [0, 2, 0, -16, -4, 0, 0, 0], [0, 28, 0, 16, -4, 0, 2, 0], [-4, 0, 12, 0, 0, 2, 0, 14], [0, 19, 0, 7, 16, 0, -20, 0], [-11, 0, -9, 0, 0, 8, 0, 8], [0, -18, 0, -24, 0, 0, -6, 0], [-16, 0, 0, 0, 0, 12, 0, 6], [0, 2, 0, 8, -8, 0, -18, 0], [17, 0, 1, 0, 0, 0, 0, -6], [0, 22, 0, 16, -4, 0, 2, 0], [18, 0, -4, 0, 0, 8, 0, 8], [16, 0, -8, 0, 0, 18, 0, 0], [-4, 0, 0, 0, 0, -22, 0, -4], [18, 0, 28, 0, 0, -4, 0, -4], [27, 0, 3, 0, 0, -4, 0, 2], [0, 24, 0, 12, 8, 0, -22, 0], [0, -4, 0, -10, 10, 0, 12, 0], [-30, 0, 4, 0, 0, -2, 0, 4], [-18, 0, -15, 0, 0, 22, 0, 10], [9, 0, 1, 0, 0, 16, 0, -2], [0, -8, 0, 4, -4, 0, -4, 0], [0, -14, 0, -2, -10, 0, 6, 0], [3, 0, 3, 0, 0, 0, 0, -15], [-3, 0, 3, 0, 0, -22, 0, 5], [43, 0, -1, 0, 0, 4, 0, -2], [0, -24, 0, 18, 6, 0, 10, 0], [0, 26, 0, -1, 9, 0, 13, 0], [0, -13, 0, -1, 7, 0, 18, 0], [-22, 0, 8, 0, 0, -30, 0, 6], [-6, 0, -1, 0, 0, -16, 0, 8], [0, 14, 0, -4, -12, 0, 16, 0], [20, 0, 20, 0, 0, -16, 0, -4], [14, 0, 24, 0, 0, 2, 0, -4], [0, -16, 0, -4, 2, 0, -16, 0], [0, -30, 0, 0, 8, 0, 8, 0], [-2, 0, 7, 0, 0, -15, 0, -15], [0, 32, 0, 20, 0, 0, 4, 0], [0, 20, 0, -10, 2, 0, 2, 0], [-18, 0, -8, 0, 0, 2, 0, -4], [0, -14, 0, -2, 12, 0, 12, 0], [-2, 0, 6, 0, 0, 4, 0, -14], [-11, 0, -29, 0, 0, -2, 0, 1], [4, 0, -8, 0, 0, 6, 0, -12], [0, 8, 0, -25, 3, 0, 5, 0], [0, 17, 0, -7, -1, 0, 14, 0], [10, 0, 5, 0, 0, -2, 0, -2], [0, 30, 0, 6, -4, 0, 22, 0], [26, 0, 6, 0, 0, -28, 0, -4], [1, 0, 1, 0, 0, 4, 0, 16], [22, 0, -4, 0, 0, -22, 0, 8], [0, -12, 0, 0, -8, 0, 2, 0], [0, -4, 0, -10, 8, 0, 20, 0], [0, -25, 0, 11, 0, 0, 12, 0], [0, 24, 0, -6, -12, 0, 0, 0], [0, -10, 0, -19, 1, 0, 21, 0], [-22, 0, 0, 0, 0, -6, 0, 0], [6, 0, -1, 0, 0, -12, 0, 0], [0, 24, 0, -18, -8, 0, -8, 0], [0, -34, 0, -10, -2, 0, 0, 0], [0, 14, 0, 2, 16, 0, 2, 0], [10, 0, -2, 0, 0, -14, 0, 10], [0, -15, 0, -3, -9, 0, -18, 0], [0, 1, 0, 13, 6, 0, -4, 0], [14, 0, -8, 0, 0, -2, 0, -2], [0, -26, 0, -14, 12, 0, -12, 0], [0, -30, 0, -3, -1, 0, -29, 0], [0, -2, 0, 4, 12, 0, -24, 0], [0, -8, 0, -26, 10, 0, 0, 0], [-25, 0, -5, 0, 0, 0, 0, -15], [18, 0, 22, 0, 0, -6, 0, 0], [10, 0, 2, 0, 0, -18, 0, 0], [16, 0, -20, 0, 0, 0, 0, -6], [0, -15, 0, 21, -4, 0, 0, 0], [0, -12, 0, 6, -4, 0, -2, 0], [44, 0, 12, 0, 0, -6, 0, 0], [0, 20, 0, -7, -9, 0, 13, 0], [-10, 0, 8, 0, 0, 16, 0, 10], [16, 0, 0, 0, 0, -30, 0, -12], [-18, 0, -7, 0, 0, 14, 0, 14], [0, 22, 0, 16, -4, 0, 12, 0], [0, 14, 0, 23, 2, 0, -14, 0], [-66, 0, -3, 0, 0, 13, 0, 1], [-44, 0, 12, 0, 0, -2, 0, 4], [0, -42, 0, 6, 2, 0, 16, 0], [-6, 0, -19, 0, 0, -19, 0, 5], [6, 0, 3, 0, 0, -12, 0, 12], [0, -18, 0, 0, -8, 0, 28, 0], [0, 24, 0, 0, 12, 0, 10, 0], [-8, 0, -6, 0, 0, 8, 0, -10], [0, 36, 0, 21, 3, 0, 9, 0], [0, 9, 0, 9, 9, 0, -6, 0], [0, -33, 0, 3, 4, 0, 20, 0], [-22, 0, -8, 0, 0, 20, 0, 2], [0, -39, 0, -15, 15, 0, -22, 0], [0, 28, 0, 4, -8, 0, -12, 0], [0, 35, 0, 11, 10, 0, -24, 0], [-35, 0, -7, 0, 0, 0, 0, 9], [0, 42, 0, 6, -2, 0, 14, 0], [-14, 0, -4, 0, 0, -34, 0, -10]]
